from xiaoaitts import XiaoAiError, ErrorCode
from xiaoaitts.miot.get_device import get_device
from xiaoaitts.miot.get_device_spec import get_device_spec


def get_my_device_spec(ticket):
    def get_current_device():
        devices = get_device(ticket)
        for device in devices:
            if device['did'] == ticket.miot_did:
                return device
        raise XiaoAiError(ErrorCode.NO_DEVICE, ticket.miot_did)

    current_device = get_current_device()
    miot_specs = get_device_spec()
    for model, _type in miot_specs.items():
        if model == current_device['model']:
            return get_device_spec(_type)
    raise XiaoAiError(ErrorCode.NO_DEVICE, ticket.miot_did)



from xiaoaitts.lib.invoke import miio


def get_device(ticket, name=None, virtual_model=False, huami_devices=0):
    data = {
        'getVirtualModel': bool(virtual_model),
        'getHuamiDevices': int(huami_devices)
    }
    devices = miio(ticket, uri='/home/device_list', data=data)['list']
    return [device for device in devices if not name or name in device['name']]



from xiaoaitts.const import API
from xiaoaitts.lib.invoke import invoke


def get_playlist_songs(ticket, list_id, count=20, offset=0):
    data = {
        'listId': list_id,
        'count': count,
        'offset': offset,
    }
    return invoke(url=API.PLAYLIST_SONGS, data=data, cookies=ticket.cookies)



import json

from xiaoaitts.lib.invoke import ubus


def get_status(ticket):
    data = ubus(ticket, message={}, method='player_get_play_status', path='mediaplayer')
    return json.loads(data['info'])



from xiaoaitts.lib.invoke import ubus


def set_play_loop(ticket, type=1):
    """
    :param ticket:
    :param type: 0-单曲循环，1-列表循环，3-列表随机
    :return:
    """
    message = {
        'type': type,
    }
    return ubus(ticket, message=message, method='player_set_loop', path='mediaplayer')



from xiaoaitts.lib.invoke import miot
from xiaoaitts.miot import get_my_device_spec


def tts(ticket, text):
    def get_service_iid():
        device_spec = get_my_device_spec(ticket)
        for service in device_spec['services']:
            if 'actions' not in service:
                continue
            for action in service['actions']:
                if 'action:play-text' in action['type']:
                    return {
                        'siid': service['iid'],
                        'aiid': action['iid']
                    }
        return {'siid': 5, 'aiid': 1}

    params = {
        'did': ticket.miot_did,
        'in': [text],
        **get_service_iid()
    }
    return miot(ticket, cmd='action', params=params)



APP_DEVICE_ID = '3C861A5820190429'
SDK_VER = '3.9'
APP_UA = 'APP/com.xiaomi.mihome APPV/6.0.103 iosPassportSDK/3.9.0 iOS/14.4 miHSTS'
MINA_UA = 'MiHome/6.0.103 (com.xiaomi.mihome; build:6.0.103.1; iOS 14.4.0) Alamofire/6.0.103 MICO/iOSApp/appStore/6.0.103'
MIIO_UA = 'iOS-14.4-6.0.103-iPhone12,3--D7744744F7AF32F0544445285880DD63E47D9BE9-8816080-84A3F44E137B71AE-iPhone'


class API:
    MIIO = 'https://api.io.mi.com/app'
    USBS = 'https://api2.mina.mi.com/remote/ubus'
    SERVICE_AUTH = 'https://account.xiaomi.com/pass/serviceLoginAuth2'
    SERVICE_LOGIN = 'https://account.xiaomi.com/pass/serviceLogin'
    PLAYLIST = 'https://api2.mina.mi.com/music/playlist/v2/lists'
    PLAYLIST_SONGS = 'https://api2.mina.mi.com/music/playlist/v2/songs'
    SONG_INFO = 'https://api2.mina.mi.com/music/song_info'
    DEVICE_LIST = 'https://api2.mina.mi.com/admin/v2/device_list'



import base64
import hashlib
import hmac
import json
import os
import random
import string
import time

from xiaoaitts.const import API
from xiaoaitts.lib.error import ErrorCode, XiaoAiError
from xiaoaitts.lib.request import request


def invoke(url, data=None, method='GET', **kwargs):
    resp = request(url, data=data, method=method, **kwargs)
    if resp['code'] == 0:
        return resp['result'] if 'io.mi.com' in url else resp['data']
    raise XiaoAiError(ErrorCode.INVALID_RESULT, resp)


def ubus(ticket, message, method, path):
    def gen_request_id():
        return 'app_ios_' + ''.join(random.sample(string.digits + string.ascii_letters, 30))

    data = {
        'deviceId': ticket.device_id,
        'message': json.dumps(message, ensure_ascii=False),
        'method': method,
        'path': path,
        'requestId': gen_request_id()
    }
    try:
        return invoke(url=API.USBS, data=data, method='POST', cookies=ticket.cookies)
    except XiaoAiError as e:
        resp = e.response
        if resp['code'] == 101 and resp['data']:
            device_data = json.loads(resp['data']['device_data'])
            raise XiaoAiError(ErrorCode.UBUS_ERR, device_data['msg'])


def miio(ticket, uri, data):
    def gen_nonce():
        """Time based nonce."""
        nonce = os.urandom(8) + int(time.time() / 60).to_bytes(4, 'big')
        return base64.b64encode(nonce).decode()

    def gen_signed_nonce(ssecret, nonce):
        """Nonce signed with ssecret."""
        m = hashlib.sha256()
        m.update(base64.b64decode(ssecret))
        m.update(base64.b64decode(nonce))
        return base64.b64encode(m.digest()).decode()

    def gen_signature(url, signed_nonce, nonce, data):
        """Request signature based on url, signed_nonce, nonce and data."""
        sign = '&'.join([url, signed_nonce, nonce, 'data=' + data])
        signature = hmac.new(key=base64.b64decode(signed_nonce),
                             msg=sign.encode(),
                             digestmod=hashlib.sha256).digest()
        return base64.b64encode(signature).decode()

    def sign_data(uri, data, ssecurity):
        if not isinstance(data, str):
            data = json.dumps(data)
        nonce = gen_nonce()
        signed_nonce = gen_signed_nonce(ssecurity, nonce)
        signature = gen_signature(uri, signed_nonce, nonce, data)
        return {'_nonce': nonce, 'data': data, 'signature': signature}

    def prepare_data():
        ticket.cookies['PassportDeviceId'] = ticket.device_id
        return sign_data(uri, data, ticket.ssecurity)

    headers = {'x-xiaomi-protocal-flag-cli': 'PROTOCAL-HTTP2'}
    return invoke(url=API.MIIO + uri, data=prepare_data(), method='POST', headers=headers, cookies=ticket.cookies)


def miot(ticket, cmd, params):
    data = {
        'params': params
    }
    return miio(ticket=ticket, uri='/miotspec/' + cmd, data=data)



import json
import os
from functools import lru_cache

from xiaoaitts.lib.request import request

"""
https://miot-spec.org/miot-spec-v2/instances?status=all
https://miot-spec.org/miot-spec-v2/instance?type=urn:miot-spec-v2:device:speaker:0000A015:xiaomi-x08c:1
"""


@lru_cache(maxsize=None)
def get_device_spec(type=None):
    if not type or not type.startswith('urn'):
        def get_spec(all):
            if not type:
                return all
            ret = {}
            for m, t in all.items():
                if type == m:
                    return {m: t}
                elif type in m:
                    ret[m] = t
            return ret

        import tempfile
        path = os.path.join(tempfile.gettempdir(), 'xiaoaitts_miot_specs.json')
        try:
            with open(path) as f:
                result = get_spec(json.load(f))
        except:
            result = None
        if not result:
            resp = request('https://miot-spec.org/miot-spec-v2/instances?status=all', data={})
            all = {i['model']: i['type'] for i in resp['instances']}
            with open(path, 'w') as f:
                json.dump(all, f)
            result = get_spec(all)
        if len(result) != 1:
            return result
        type = list(result.values())[0]

    return request('https://miot-spec.org/miot-spec-v2/instance?type=' + type, data={})



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='xiaoaitts',
    version='0.1.1',
    author='Max',
    author_email='mr.qchao@gmail.com',
    description='小爱音箱自定义文本朗读',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/cnmax/xiaoaitts',
    project_urls={
        'Bug Tracker': 'https://github.com/cnmax/xiaoaitts/issues',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(exclude=('tests', 'tests.*')),
    python_requires='>=3.6',
    install_requires=[
        'requests',
    ]
)



from xiaoaitts.lib.invoke import ubus


def play_url(ticket, url, type=1):
    message = {
        'type': type,
        'url': url,
        'media': 'app_ios'
    }
    return ubus(ticket, message=message, method='player_play_url', path='mediaplayer')



import json

from xiaoaitts.lib.request import HttpError


class ErrorCode:
    AURH_ERR = 401,
    INVALID_INPUT = 1,
    NO_DEVICE = 2,
    UBUS_ERR = 3,
    INVALID_RESULT = 4


ERROR_CODE_MAP = {
    ErrorCode.AURH_ERR: 'Session 校验失败，请重新登录',
    ErrorCode.NO_DEVICE: '未找到在线设备，请检查设备连接',
    ErrorCode.INVALID_INPUT: '参数不合法，请查阅文档',
    ErrorCode.INVALID_RESULT: '接口错误',
    ErrorCode.UBUS_ERR: '请检查设备连接'
}


class XiaoAiError(Exception):

    def __init__(self, code, error_message='') -> None:
        if isinstance(code, HttpError):
            if code.status in ERROR_CODE_MAP:
                message = ERROR_CODE_MAP[code.status]
            else:
                message = '网络请求错误'
                error_message = code.message
        elif code == ErrorCode.INVALID_RESULT:
            self.response = error_message
            message = ERROR_CODE_MAP[code]
        else:
            message = ERROR_CODE_MAP.get(code, '')

        if not isinstance(error_message, str):
            error_message = json.dumps(error_message)
        self.message = error_message + (' - ' + message if message else '')

    def __str__(self) -> str:
        return self.message
